#ifdef MAKE_XMMS
#include <xmms/util.h>
#include <xmms/configfile.h>
#endif

#ifdef MAKE_AUDACIOUS
#include <audacious/util.h>
#include <audacious/configdb.h>
#define ConfigFile ConfigDb
#define xmms_cfg_open_default_file bmp_cfg_db_open
#define xmms_cfg_read_string bmp_cfg_db_get_string
#define xmms_cfg_write_string bmp_cfg_db_set_string
#define xmms_cfg_free bmp_cfg_db_close
#endif

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include "config.h"

static GtkWidget *eduname, *edpwd, *edcode;

static int errorbox_done;

void
mixi_about_show(void)
{
  static GtkWidget *aboutbox;
  gchar *tmp;

  if (aboutbox) return;

#ifdef MAKE_XMMS
  tmp = g_strdup_printf("XMMS mixi Music Plugin %s\n\n"
						"Created by Audun Hove <audun@nlc.no> and Pipian <pipian@pipian.com>\n", 
						VERSION);

  aboutbox = xmms_show_message("About XMMS-mixi-Music",
							   tmp,
							   "OK", FALSE, NULL, NULL);
#endif

#ifdef MAKE_AUDACIOUS
  tmp = g_strdup_printf("Audacious mixi Music Plugin %s\n\n"
						"Created by Audun Hove <audun@nlc.no> and Pipian <pipian@pipian.com>\n", 
						VERSION);
  aboutbox = xmms_show_message("About Audacious-mixi-Music",
							   tmp,
							   "OK", FALSE, NULL, NULL);
#endif

  g_free(tmp);
  gtk_signal_connect(GTK_OBJECT(aboutbox), "destroy",
					 GTK_SIGNAL_FUNC(gtk_widget_destroyed), &aboutbox);
}

static void
set_errorbox_done(GtkWidget *errorbox, GtkWidget **errorboxptr)
{
  errorbox_done = -1;
  gtk_widget_destroyed(errorbox, errorboxptr);
}

void
mixi_init_errorbox_done(void)
{
  errorbox_done = 1;
}

int
mixi_get_errorbox_done(void)
{
  return errorbox_done;
}

void
mixi_errorbox_show(char *errortxt)
{
  static GtkWidget *errorbox;
  gchar *tmp;

  if(errorbox_done != 1) return;

  errorbox_done = 0;
  tmp = g_strdup_printf("There has been an error"
						" that may require your attention.\n\n"
						"Contents of server error:\n\n"
						"%s\n",
						errortxt);

#ifdef MAKE_XMMS
  errorbox = xmms_show_message("XMMS-mixi-Music Error",
							   tmp,
							   "OK", FALSE, NULL, NULL);
#endif

#ifdef MAKE_AUDACIOUS
  errorbox = xmms_show_message("Audacious-mixi-Music Error",
							   tmp,
							   "OK", FALSE, NULL, NULL);
#endif

  g_free(tmp);
  gtk_signal_connect(GTK_OBJECT(errorbox), "destroy",
					 GTK_SIGNAL_FUNC(set_errorbox_done), &errorbox);
}


/*
static char *
hexify(char *pass, int len)
{
	static char buf[33];
	char *bp = buf;
	char hexchars[] = "0123456789abcdef";
	int i;
	
	memset(buf, 0, sizeof(buf));
	
	for(i = 0; i < len; i++) {
		*(bp++) = hexchars[(pass[i] >> 4) & 0x0f];
		*(bp++) = hexchars[pass[i] & 0x0f];
	}
	*bp = 0;
	return buf;
}
*/

static void
saveconfig(GtkWidget *wid, gpointer data)
{
  ConfigFile *cfgfile;

  const char *pwd = gtk_entry_get_text(GTK_ENTRY(edpwd));
  const char *uid = gtk_entry_get_text(GTK_ENTRY(eduname));
  char *code = strdup(gtk_entry_get_text(GTK_ENTRY(edcode)));

  if ((cfgfile = xmms_cfg_open_default_file())) {
	
	xmms_cfg_write_string(cfgfile, "miximusic", "email", (char *)uid);

	if (pwd != NULL && pwd[0] != '\0')
	  xmms_cfg_write_string(cfgfile, "miximusic", "password", (char *)pwd);

	g_strdelimit(code, " ,:;|/", ',');

	xmms_cfg_write_string(cfgfile, "miximusic", "mp3tagcode", (char *)code);

	g_free(code);

#ifdef MAKE_XMMS
	xmms_cfg_write_default_file(cfgfile);
#endif
	xmms_cfg_free(cfgfile);
  }
  gtk_widget_destroy(GTK_WIDGET(data));
}

void
mixi_configure_dialog(void)
{
  static GtkWidget *cnfdlg;
  GtkWidget	*btnok, *btncancel, *vbox, *hbox, *unhbox,
	*pwhbox, *cdhbox, *lblun, *lblpw, *lblcd, *frame;

  ConfigFile *cfgfile;
	
  if (cnfdlg) return;

#ifdef MAKE_XMMS
  cnfdlg = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_set_title(GTK_WINDOW(cnfdlg),
					   "XMMS-mixi-Music configuration");
#endif

#ifdef MAKE_AUDACIOUS
  cnfdlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_type_hint(GTK_WINDOW(cnfdlg),
						   GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_title(GTK_WINDOW(cnfdlg),
					   "Audacious-mixi-Music configuration");
#endif

  gtk_signal_connect(GTK_OBJECT(cnfdlg), "destroy",
					 GTK_SIGNAL_FUNC(gtk_widget_destroyed), &cnfdlg);
	
  vbox = gtk_vbox_new(FALSE, 0);

  unhbox = gtk_hbox_new(FALSE, 0);
  eduname = gtk_entry_new();
  lblun = gtk_label_new("Email");
  gtk_box_pack_start(GTK_BOX(unhbox), lblun, FALSE, FALSE, 3);
  gtk_box_pack_start(GTK_BOX(unhbox), eduname, FALSE, FALSE, 3);
	
  pwhbox = gtk_hbox_new(FALSE, 0);
  edpwd = gtk_entry_new();
  lblpw = gtk_label_new("Password");
  gtk_entry_set_visibility(GTK_ENTRY(edpwd), FALSE);
  gtk_box_pack_start(GTK_BOX(pwhbox), lblpw, FALSE, FALSE, 3);
  gtk_box_pack_start(GTK_BOX(pwhbox), edpwd, FALSE, FALSE, 3);

  cdhbox = gtk_hbox_new(FALSE, 0);
  edcode = gtk_entry_new();
  lblcd = gtk_label_new("MP3 Tag Code");
  gtk_box_pack_start(GTK_BOX(cdhbox), lblcd, FALSE,FALSE, 3);
  gtk_box_pack_start(GTK_BOX(cdhbox), edcode, FALSE,FALSE, 3);
  gtk_box_pack_start(GTK_BOX(vbox), unhbox, FALSE, FALSE, 3);
  gtk_box_pack_start(GTK_BOX(vbox), pwhbox, FALSE, FALSE, 3);
  gtk_box_pack_start(GTK_BOX(vbox), cdhbox, FALSE, FALSE, 3);
	
  hbox = gtk_hbox_new(FALSE, 0);

  btnok = gtk_button_new_with_label("OK");
  gtk_signal_connect(GTK_OBJECT(btnok), "clicked",
					 GTK_SIGNAL_FUNC(saveconfig), GTK_OBJECT(cnfdlg));
	
  btncancel = gtk_button_new_with_label("Cancel");
  gtk_signal_connect_object(GTK_OBJECT(btncancel), "clicked",
							GTK_SIGNAL_FUNC(gtk_widget_destroy),
							GTK_OBJECT(cnfdlg));

  gtk_box_pack_start(GTK_BOX(hbox), btnok, FALSE, FALSE, 3);
  gtk_box_pack_start(GTK_BOX(hbox), btncancel, FALSE, FALSE, 3);
	
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);

  frame = gtk_frame_new(" The plugin will have to be restarted for changes to take effect! ");

  gtk_container_add(GTK_CONTAINER(frame), vbox);
  gtk_container_add(GTK_CONTAINER(cnfdlg), frame);

  if ((cfgfile = xmms_cfg_open_default_file())) {
	gchar *email = NULL;
	gchar *mp3tagcode = NULL;

	xmms_cfg_read_string(cfgfile, "miximusic", "email", &email);
	if (email) {
	  gtk_entry_set_text(GTK_ENTRY(eduname), email);
	  g_free(email);
	}

	xmms_cfg_read_string(cfgfile, "miximusic", "mp3tagcode", &mp3tagcode);
	if (mp3tagcode) {
	  gtk_entry_set_text(GTK_ENTRY(edcode), mp3tagcode);
	  g_free(mp3tagcode);
	}

	xmms_cfg_free(cfgfile);
  }
	
  gtk_widget_show_all(cnfdlg);
}
