#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include "tags/include/tags.h"
#include "queue.h"
#include "fmt.h"

static item_t *q_queue = NULL;
static item_t *q_queue_last = NULL;
static int q_nitems = 0;

static void
q_item_free(item_t *item)
{
  if (item == NULL){
	pdebug("Warning: NULL item", DEBUG);
	return;
  }

  curl_free(item->artist);
  curl_free(item->title);
  curl_free(item->utctime);
  curl_free(item->genre);
  curl_free(item->album);
  free(item);
}


void
mixi_q_put(metatag_t *meta, int len)
{
  item_t *item;

  item = malloc(sizeof(item_t));

  item->artist = curl_escape(meta->artist, 0);
  item->title = curl_escape(meta->title, 0);
  item->utctime = curl_escape(mixi_fmt_timestr(time(NULL), 0), 0);

  item->genre = meta->genre ? curl_escape(meta->genre, 0) : NULL;
	
  snprintf(item->len, sizeof(item->len), "%d", len);

  pdebug(mixi_fmt_vastr("item->title: %s", item->title), DEBUG);

  if (meta->album == NULL)
	item->album = curl_escape("", 0);
  else
	item->album = curl_escape(meta->album, 0);

  q_nitems++;

  item->next = NULL;

  if (q_queue_last == NULL) {
	q_queue = q_queue_last = item; 
  } else {
	q_queue_last->next = item;
	q_queue_last = item;
  }
}


item_t *
mixi_q_put2(char *artist, char *title, char *len, char *time, 
			char *album, char *genre)
{
  char *temp = NULL;

  item_t *item;

  item = calloc(1, sizeof(item_t));

  temp = curl_unescape(artist, 0);
  item->artist = curl_escape(temp, 0);
  curl_free(temp);

  temp = curl_unescape(title, 0);
  item->title = curl_escape(temp, 0);
  curl_free(temp);

  memcpy(item->len, len, sizeof(len));

  temp = curl_unescape(time, 0);
  item->utctime = curl_escape(temp, 0);
  curl_free(temp);

  temp = curl_unescape(album, 0);
  item->album = curl_escape(temp, 0);
  curl_free(temp);

  temp = curl_unescape(genre, 0);
  item->genre = curl_escape(temp, 0);
  curl_free(temp);

  q_nitems++;

  item->next = NULL;

  if (q_queue_last == NULL) {
	q_queue = q_queue_last = item; 
  } else {
	q_queue_last->next = item;
	q_queue_last = item;
  }

  return item;
}



item_t *
mixi_q_peek(void)
{
  if (!q_nitems){	
	pdebug("Warning: queue is empty", DEBUG);
	return NULL;
  }
  
  return q_queue; 
}


item_t *
mixi_q_peekall(int rewind)
{
  static item_t *citem = NULL; 

  item_t *temp_item;

  if (rewind) {
	citem = q_queue;
	return NULL;
  }

  temp_item = citem;
  
  if(citem != NULL)
	citem = citem->next;

  return temp_item;
}


int 
mixi_q_get(void)
{
  item_t *item;

  if ( q_nitems == 0 ) {
	//q_queue_last = NULL;
	pdebug("Warning: queue is empty", DEBUG);
	return 0;
  }

  item = q_queue; 

  if(item == NULL){
	pdebug("Warning: queue is empty", DEBUG);
	return 0; 
  }

  q_nitems --; 
  q_queue = q_queue->next; 

  q_item_free(item); 

  if ( q_nitems == 0 ) {
	q_queue_last = NULL;
  }

  return -1;
}

void 
mixi_q_free(void)
{
  while (mixi_q_get());
}

int 
mixi_q_len(void)
{
  return q_nitems;
}
