/*
 *  Copyright (C) 2006 Konosuke Watanabe <sasugaanija[at]gmail.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <glib.h>

#include "config.h"
#include "../fmt.h"

#ifdef HAVE_ICONV
#include <iconv.h>
#endif

#ifdef MAKE_XMMS
char *
mixi_convert_to_utf8(char *str, char **froms)
{

#ifdef HAVE_ICONV
  iconv_t cd;
  char **from = froms, *outbuf;
  char *routbuf = NULL;
  size_t inbytes, outbytes;
#endif

  if (str == NULL) return NULL;

  if (*str == '\0')	return strdup(str);

#ifdef HAVE_ICONV
  outbytes = strlen(str) * 3;
  routbuf = malloc( outbytes + 1 ); 

  while (*from) {

	if (**from == '\0') {
	  from ++;
	  continue;
	}

	inbytes = strlen(str);
	outbuf = routbuf;

	cd = iconv_open("UTF-8", *from);
	if (cd == (iconv_t)-1) {
	  pdebug(mixi_fmt_vastr("iconv_open: failed (%s to UTF-8)", *from), DEBUG);
	  from ++;
	  continue;
	}
	

	if (iconv(cd, &str, &inbytes, &outbuf, &outbytes) == -1) {
	  pdebug(mixi_fmt_vastr("iconv: failed (%s to UTF-8)", *from), DEBUG);
	  switch (errno){
	  case E2BIG:
		pdebug("There is not sufficient room at outbuf.", DEBUG);
		break;
	  case EINVAL:
		pdebug("An incomplete multibyte sequence has been encountered in the input.", DEBUG);
		break;
	  case EILSEQ:
		pdebug("An invalid multibyte sequence has been encountered in the input.", DEBUG);
		break;
	  default:
		pdebug("Unkonw error.", DEBUG);
		break;
	  }
	  from ++;
	  iconv_close(cd);
	  continue;
	}
	*outbuf = '\0';

	iconv_close(cd);

	return routbuf;
  }

  return NULL;

#else // HAVE_ICONV
  return strdup(str); 
#endif // HAVE_ICONV
}
#endif // MAKE_XMMS


#ifdef MAKE_AUDACIOUS
char *
mixi_convert_to_utf8(char *str, char **froms)
{
  char **from = froms;
  char *routbuf = NULL;

  while (*from) {
	if (**from != '\0') {
	  if ((routbuf = g_convert(str, strlen(str), "UTF-8", *from,
							   NULL, NULL, NULL)) == NULL) {
		// failed
		pdebug(mixi_fmt_vastr("g_convert: failed (%s)", *from), DEBUG);
	  } else {
		return routbuf;
	  }
	}
	from ++;
  }
  return routbuf;
}

#endif
